/***************************************************************************
 *   Copyright (C) 2006 by Martin Strasser                                 *
 *   strasser  a t  cadmaniac  d o t  org                                  *
 *   Special thanks to Mario Boikov                                        *
 *   squeeze  a t  cadmaniac  d o t  org                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "klsevensegmentwidget.h"
#include <qtoolbutton.h>
#include <qlistview.h>
#include <qpainter.h>
#include <qpixmap.h>
#include <qlineedit.h>
#include "kldocument.h"
#include "kldocumentview.h"

#define WIZARD_START "// WIZARD CODE"
#define WIZARD_END   "// END OF WIZARD CODE"

KLSevenSegmentWidget::KLSevenSegmentWidget(QWidget *parent, const char *name, KLDocument* doc)
    :KLSevenSegmentWidgetBase(parent, name)
{
    QPtrList< QRadioButton > helper;
    helper.setAutoDelete( false );
    helper.append( rbA0 );
    helper.append( rbA1 );
    helper.append( rbA2 );
    helper.append( rbA3 );
    helper.append( rbA4 );
    helper.append( rbA5 );
    helper.append( rbA6 );
    helper.append( rbA7 );
    m_radios.append( helper );
    helper.clear();
    helper.append( rbB0 );
    helper.append( rbB1 );
    helper.append( rbB2 );
    helper.append( rbB3 );
    helper.append( rbB4 );
    helper.append( rbB5 );
    helper.append( rbB6 );
    helper.append( rbB7 );
    m_radios.append( helper );
    helper.clear();
    helper.append( rbC0 );
    helper.append( rbC1 );
    helper.append( rbC2 );
    helper.append( rbC3 );
    helper.append( rbC4 );
    helper.append( rbC5 );
    helper.append( rbC6 );
    helper.append( rbC7 );
    m_radios.append( helper );
    helper.clear();
    helper.append( rbD0 );
    helper.append( rbD1 );
    helper.append( rbD2 );
    helper.append( rbD3 );
    helper.append( rbD4 );
    helper.append( rbD5 );
    helper.append( rbD6 );
    helper.append( rbD7 );
    m_radios.append( helper );
    helper.clear();
    helper.append( rbE0 );
    helper.append( rbE1 );
    helper.append( rbE2 );
    helper.append( rbE3 );
    helper.append( rbE4 );
    helper.append( rbE5 );
    helper.append( rbE6 );
    helper.append( rbE7 );
    m_radios.append( helper );
    helper.clear();
    helper.append( rbF0 );
    helper.append( rbF1 );
    helper.append( rbF2 );
    helper.append( rbF3 );
    helper.append( rbF4 );
    helper.append( rbF5 );
    helper.append( rbF6 );
    helper.append( rbF7 );
    m_radios.append( helper );
    helper.clear();
    helper.append( rbG0 );
    helper.append( rbG1 );
    helper.append( rbG2 );
    helper.append( rbG3 );
    helper.append( rbG4 );
    helper.append( rbG5 );
    helper.append( rbG6 );
    helper.append( rbG7 );
    m_radios.append( helper );
    helper.clear();
    helper.append( rbDP0 );
    helper.append( rbDP1 );
    helper.append( rbDP2 );
    helper.append( rbDP3 );
    helper.append( rbDP4 );
    helper.append( rbDP5 );
    helper.append( rbDP6 );
    helper.append( rbDP7 );
    m_radios.append( helper );
    helper.clear();
    m_document = doc;
    m_dontReactOnToolButtons=false;
    // Here starts the reread code:
    m_whiteSpace = "";
    
    m_startLine = m_stopLine = -1;
    // Now we search in the document for the start and the stop, if any:
    if ( m_document->lastActiveView() )
    {
        if ( m_document->lastActiveView()->view() )
        {
            int curLine = m_document->lastActiveView()->view()->cursorLine();
            m_stopLine = m_startLine = curLine;
            bool foundStart = false, foundStop = false;
            while ( m_startLine >= 0 )
            {
                if ( m_startLine != curLine )
                    if (m_document->editIf->textLine( m_startLine ).upper().stripWhiteSpace().startsWith( WIZARD_END ))
                        break;
                if (!m_document->editIf->textLine( m_startLine ).upper().stripWhiteSpace().startsWith( WIZARD_START ))
                    m_startLine--;
                else
                {
                    foundStart = true;
                    break;
                }
            }
            while ( m_stopLine < (int) m_document->editIf->numLines() )
            {
                if ( m_stopLine != curLine )
                    if (m_document->editIf->textLine( m_stopLine ).upper().stripWhiteSpace().startsWith( WIZARD_START ))
                        break;
                if (!m_document->editIf->textLine( m_stopLine ).upper().stripWhiteSpace().startsWith( WIZARD_END ))
                    m_stopLine++;
                else
                {
                    foundStop = true;
                    break;
                }
            }
            if (!( foundStart && foundStop ))
                m_startLine = m_stopLine = -1;
        }
    }
    
    QRadioButton *bit;
    QPtrList< QRadioButton > seg;
    for ( unsigned int i=0; i<m_radios.count(); i++ )
    {
        seg=m_radios[i];
        for ( bit=seg.first(); bit; bit=seg.next() )
        {
            connect( bit, SIGNAL(toggled( bool )), this, SLOT(slotBitSegmentAssocChanged()) );
        }
    }
    // Now set the bit to segment assignment list.
    // The meaning is: m_segmentIsBit[2] gives the bit mask (not bit number) for segment C
    
    m_segmentIsBit.append(0x1);
    m_segmentIsBit.append(0x2);
    m_segmentIsBit.append(0x4);
    m_segmentIsBit.append(0x8);
    m_segmentIsBit.append(0x10);
    m_segmentIsBit.append(0x20);
    m_segmentIsBit.append(0x40);
    m_segmentIsBit.append(0x80);
    
    if ( underReedit() )
    {
        // Seek the line with the // WIZARD SEVEN_SEGMENT_WIZARD in it.
        int curLine = m_startLine;
        bool ok;
        while ( curLine < m_stopLine )
        {
            curLine++;
            if ( m_document->editIf->textLine( curLine ).upper().stripWhiteSpace().startsWith( "// WIZARD SEVEN_SEGMENT_WIZARD" ) )
            {
                QString bitAssignmentString = m_document->editIf->textLine( curLine ).upper().stripWhiteSpace();
                QStringList slist = QStringList::split( " ", bitAssignmentString );
                bitAssignmentString = slist[3];
                for (int i=0; i<8; i++)
                {
                    int segmentNr = bitAssignmentString.mid(i,1).toInt( &ok );
                    if (!ok)
                        break;
                    m_segmentIsBit[segmentNr] = 1<<i;
                }
                break;
            }
        }
        curLine++;
        QString allLinesBelow;
        while (curLine < m_stopLine)
        {
            int ws = 0;
            QString current = m_document->editIf->textLine( curLine );
            if ( current.contains( "=" ) )
            {
                ws = current.left( current.find("=")+1 ).length() -
                        current.left( current.find("=")+1 ).stripWhiteSpace().length();
                m_whiteSpace = current.left( ws );
            }
            allLinesBelow += current.stripWhiteSpace();
            curLine++;
        }
        QString theName = allLinesBelow.mid( 0, allLinesBelow.find( "=" ) ).stripWhiteSpace();
        theName = theName.right( theName.length() - theName.findRev( " " ) );
        theName = theName.replace("[", "").replace("]", "");
        leArrayName->setText( theName.stripWhiteSpace() );
        allLinesBelow = allLinesBelow.mid( allLinesBelow.find( "{" )+1,
                                           allLinesBelow.findRev( "}" ) - allLinesBelow.find( "{" ) - 1 );
        allLinesBelow = allLinesBelow.replace( ",", " " );
        QStringList list = QStringList::split(" ", allLinesBelow, false);
        QValueList< int > segs, sortedSegs;
        for ( QStringList::Iterator it=list.begin(); it != list.end(); ++it )
        {
            if ( (*it).upper().startsWith("0X") )
            {
                int addMe = (*it).right((*it).length()-2).toInt(&ok, 16);
                if (!ok)
                    break;
                segs.append( addMe );
            }
            else
            {
                int addMe = (*it).toInt(&ok);
                if (!ok)
                    break;
                segs.append( addMe );
            }
        }
        for ( unsigned int i=0; i<segs.size(); i++ )
        {
            int addMe = 0;
            for ( int j=0; j<8; j++ )
            {
                if ( segs[i] & m_segmentIsBit[j] )
                        addMe |= 1<<j;
            }
            sortedSegs.append( addMe );
        }
        if ( sortedSegs.size() > 0 )
        {
            lvSegments->clear();
            QListViewItem *item;
            for ( unsigned int i=0; i<sortedSegs.size(); i++ )
            {
                item = new QListViewItem(lvSegments, formStr( i ) );
                for ( int j=0; j<8; j++ )
                    item->setText( j+1, (1<<j)&sortedSegs[i] ? "1": "0" );
                item->setPixmap( 0, generatePixmapFor( getSegmentsForListItem( item ) ) );
                lvSegments->insertItem( item );
            }
        }
    }
    m_prohibitRecursion=true;
    setRadiosFromBitSegmentAssignment( m_segmentIsBit );
    m_prohibitRecursion=false;
    lvSegments->setSelected( lvSegments->firstChild(), true );
    slotCurrentItemChanged( lvSegments->firstChild() );
    
    QListViewItem * myChild = lvSegments->firstChild();
    while( myChild ) {
        myChild->setPixmap( 0, generatePixmapFor( getSegmentsForListItem( myChild ) ) );
        myChild = myChild->nextSibling();
    }
    lvSegments->setSorting(0);
}

void KLSevenSegmentWidget::slotBitSegmentAssocChanged()
{
    if ( m_prohibitRecursion )
        return;
    m_prohibitRecursion=true;
    QValueList< int > newMask;

    newMask = readBitSegmentAssignmentFromRadios();
    // Find out the row which has changed:
    int changedRow=-1;
    
    for ( unsigned int i=0; i<newMask.count(); i++ )
    {
        if ( newMask[i] != m_segmentIsBit[i] )
        {
            changedRow=i;
            break;
        }
    }
    if ( changedRow<0 )
    {
        qWarning("ERROR: changedRow<0 %d", changedRow);
    }
    
    // Check if the user deactivated the previously checked radio button:
    if ( newMask[changedRow] == 0 )
    {
        // Set the old mask and exit:
        setRadiosFromBitSegmentAssignment( m_segmentIsBit );
        m_prohibitRecursion = false;
        return;
    }
    // Find out, how the mask should be set according to the user:
    int maskShouldBe = newMask[changedRow] & ( newMask[changedRow] ^ m_segmentIsBit[changedRow] );
    // Check which mask has to be changed now!
    int neccessarilyChangeRow = -1;
    for ( unsigned int i=0; i<newMask.count(); i++ )
    {
        if ( m_segmentIsBit[i] == maskShouldBe )
        {
            neccessarilyChangeRow = i;
            break;
        }
    }
    if ( neccessarilyChangeRow<0 )
    {
        qWarning("ERROR: neccessarilyChangeRow<0 %d", neccessarilyChangeRow);
    }
    // Set the neccessarily changed row to the mask which was set in the changedRow before:
    newMask[neccessarilyChangeRow] = m_segmentIsBit[changedRow];
    // Then set the new mask in the changedRow:
    newMask[changedRow] = maskShouldBe;
    setRadiosFromBitSegmentAssignment( newMask );
    // These should stay the last 2 assignments:
    m_prohibitRecursion = false;
    m_segmentIsBit = readBitSegmentAssignmentFromRadios();
}


void KLSevenSegmentWidget::slotDP()
{
    tbDP->setPalette( tbDP->isOn() ? QPalette( Qt::red, Qt::red ) : palette() );
    changeCurrentListItem();
}


void KLSevenSegmentWidget::slotG()
{
    tbG->setPalette( tbG->isOn() ? QPalette( Qt::red, Qt::red ) : palette() );
    changeCurrentListItem();
}


void KLSevenSegmentWidget::slotF()
{
    tbF->setPalette( tbF->isOn() ? QPalette( Qt::red, Qt::red ) : palette() );
    changeCurrentListItem();
}


void KLSevenSegmentWidget::slotE()
{
    tbE->setPalette( tbE->isOn() ? QPalette( Qt::red, Qt::red ) : palette() );
    changeCurrentListItem();
}


void KLSevenSegmentWidget::slotD()
{
    tbD->setPalette( tbD->isOn() ? QPalette( Qt::red, Qt::red ) : palette() );
    changeCurrentListItem();
}


void KLSevenSegmentWidget::slotC()
{
    tbC->setPalette( tbC->isOn() ? QPalette( Qt::red, Qt::red ) : palette() );
    changeCurrentListItem();
}


void KLSevenSegmentWidget::slotB()
{
    tbB->setPalette( tbB->isOn() ? QPalette( Qt::red, Qt::red ) : palette() );
    changeCurrentListItem();
}


void KLSevenSegmentWidget::slotA()
{
    tbA->setPalette( tbA->isOn() ? QPalette( Qt::red, Qt::red ) : palette() );
    changeCurrentListItem();
}


void KLSevenSegmentWidget::slotCancel()
{
    close();
}


void KLSevenSegmentWidget::slotOK()
{
    QValueList< int > bitmasks = generateBitmasks();
    QString varDef;
    QString lineStart;
    QString out(WIZARD_START);
    out += " Do not edit the lines below.\n";
    out += "// Otherwise the wizard may be unable to reread the settings.\n";
    out += "// WIZARD SEVEN_SEGMENT_WIZARD ";
    for (int i=0; i<8; i++)
    {
        for (int j=0; j<8; j++)
        {
            if (m_segmentIsBit[j] == (1<<i))
                out += QString("%1").arg(j);
        }
    }
    out += "\n";
    varDef = "char " + leArrayName->text() + "[] = {";
    lineStart = QString("%1").arg( " ", varDef.length() );
    out += varDef;
    int counter=0;
    for (unsigned int i=0; i<bitmasks.count(); i++)
    {
        out += " " + QString("0x%1").arg(bitmasks[i], 0, 16) + ",";
        counter++;
        if (counter > 6)
        {
            counter=0;
            out += "\n"+lineStart;
        }
    }
    out = out.left( out.length()-1 );
    out += " };\n";
    out += WIZARD_END;
    out += "\n";
    int line=0;
    if (m_document->lastActiveView())
        if (m_document->lastActiveView()->view())
            line=m_document->lastActiveView()->view()->cursorLine();
    QStringList lines;
    lines = lines.split( "\n", out );
    if ( underReedit() )
    {
        for ( int curLine=m_stopLine; curLine >= m_startLine; curLine-- )
            m_document->editIf->removeLine( curLine );
        line = m_startLine;
    }
    for (QStringList::Iterator it = lines.begin(); it != lines.end(); ++it )
    {
        m_document->editIf->insertLine( line, m_whiteSpace + (*it) );
        line++;
    }
    close();
}


void KLSevenSegmentWidget::slotDown()
{
    QListViewItem *next,*item=lvSegments->currentItem();
    bool ok;
    
    next = item->nextSibling();
    if (!next)
        return;
    lvSegments->takeItem( next );
    lvSegments->takeItem( item );
    item->setText( 0, formStr( item->text( 0 ).mid(2,2).toInt( &ok, 16 )+1) );
    next->setText( 0, formStr( next->text( 0 ).mid(2,2).toInt( &ok, 16 )-1) );
    lvSegments->insertItem( next );
    lvSegments->insertItem( item );
    lvSegments->setCurrentItem( item );
}


void KLSevenSegmentWidget::slotUp()
{
    QListViewItem *prev,*item=lvSegments->currentItem();
    bool ok;
    
    prev=lvSegments->firstChild();
    if ( prev == item )
        return;
    while(prev->nextSibling()!=item)
        prev=prev->nextSibling();
    lvSegments->takeItem( prev );
    lvSegments->takeItem( item );
    item->setText( 0, formStr( item->text( 0 ).mid(2,2).toInt( &ok, 16 )-1 ) );
    prev->setText( 0, formStr( prev->text( 0 ).mid(2,2).toInt( &ok, 16 )+1 ) );
    lvSegments->insertItem( prev );
    lvSegments->insertItem( item );
    lvSegments->setCurrentItem( item );
}


void KLSevenSegmentWidget::slotRemove()
{
    if (lvSegments->childCount() == 1)
        return;
    lvSegments->takeItem( lvSegments->currentItem() );
    lvSegments->setSelected( lvSegments->currentItem(), true );
}


void KLSevenSegmentWidget::slotAdd()
{
    QListViewItem *item=lvSegments->currentItem();
    if (lvSegments->childCount() > 255)
        return;
    while(item->nextSibling())
        item=item->nextSibling();
    item = new QListViewItem(lvSegments, formStr( lvSegments->childCount() ), "0", "0", "0", "0", "0", "0", "0");
    item->setText( 9, "0" );
    item->setPixmap( 0, generatePixmapFor( getSegmentsForListItem( item ) ) );
    lvSegments->insertItem( item );
    lvSegments->setSelected( item, true );
}

void KLSevenSegmentWidget::slotCurrentItemChanged( QListViewItem* cur )
{
    int segs = getSegmentsForListItem( cur );

    m_dontReactOnToolButtons=true;
    tbA->setOn(segs&0x01);
    tbB->setOn(segs&0x02);
    tbC->setOn(segs&0x04);
    tbD->setOn(segs&0x08);
    tbE->setOn(segs&0x10);
    tbF->setOn(segs&0x20);
    tbG->setOn(segs&0x40);
    tbDP->setOn(segs&0x80);
    m_dontReactOnToolButtons=false;
}

QValueList < int > KLSevenSegmentWidget::readBitSegmentAssignmentFromRadios()
{
    QValueList < int > retVal;
    QRadioButton *bit;
    QPtrList< QRadioButton > seg;
    for ( unsigned int i=0; i<m_radios.count(); i++ )
    {
        int j=0;
        int mask=0;
        seg=m_radios[i];
        for ( bit=seg.first(); bit; bit=seg.next() )
        {
            if (bit->isChecked())
                mask|=(1<<j);
            j++;
        }
        retVal.append( mask );
    }
    if ( retVal.count() != m_radios.count() )
    {
        qWarning( "retVal.count() %d != m_radios.count() %d", retVal.count(), m_radios.count() );
    }
    return retVal;
}

void KLSevenSegmentWidget::setRadiosFromBitSegmentAssignment( QValueList < int > assign )
{
    if ( assign.count() != m_radios.count() )
    {
        qWarning( "assign.count() %d != m_radios.count() %d", assign.count(), m_radios.count() );
        return;
    }
    QRadioButton *bit;
    QPtrList< QRadioButton > seg;
    for ( unsigned int i=0; i<m_radios.count(); i++ )
    {
        int j=0;
        seg=m_radios[i];
        for ( bit=seg.first(); bit; bit=seg.next() )
        {
            bit->setChecked( assign[i] & (1<<j) );
            j++;
        }
    }
}

void KLSevenSegmentWidget::changeCurrentListItem()
{
    if (m_dontReactOnToolButtons)
        return;
    int bits = 0;
    if ( tbA->isOn() )
        bits |= 0x01;
    if ( tbB->isOn() )
        bits |= 0x02;
    if ( tbC->isOn() )
        bits |= 0x04;
    if ( tbD->isOn() )
        bits |= 0x08;
    if ( tbE->isOn() )
        bits |= 0x10;
    if ( tbF->isOn() )
        bits |= 0x20;
    if ( tbG->isOn() )
        bits |= 0x40;
    if ( tbDP->isOn() )
        bits |= 0x80;
    QListViewItem *item = lvSegments->currentItem();
    item->setPixmap( 0, generatePixmapFor( bits ) );
    setListItemForSegments( item, bits );
}

QPixmap KLSevenSegmentWidget::generatePixmapFor( int segs )
{
    int vsize=14, hsize=7;
    QPixmap retVal(hsize+2,vsize);
    retVal.fill( lvSegments->backgroundColor() );
    QPainter paint(&retVal);
    paint.setPen( Qt::red );
    if (segs&0x01)
        paint.drawLine( 1, 0, hsize-2, 0 );
    if (segs&0x02)
        paint.drawLine( hsize-1, 1, hsize-1, vsize/2-1 );
    if (segs&0x04)
        paint.drawLine( hsize-1, vsize/2+1, hsize-1, vsize-2 );
    if (segs&0x08)
        paint.drawLine( 1, vsize-1, hsize-2, vsize-1 );
    if (segs&0x10)
        paint.drawLine( 1, vsize/2+1, 1, vsize-2 );
    if (segs&0x20)
        paint.drawLine( 1, 1, 1, vsize/2-1 );
    if (segs&0x40)
        paint.drawLine( 1, vsize/2, hsize-2, vsize/2 );
    if (segs&0x80)
        paint.drawPoint( hsize+1, vsize-1 );
    return retVal;
}

int KLSevenSegmentWidget::getSegmentsForListItem( QListViewItem* item )
{
    int retVal=0;
    for (int i=0; i<8; i++)
    {
        if (item->text( i+1 ) == "1")
            retVal |= (1<<i);
    }
    return retVal;
}


void KLSevenSegmentWidget::setListItemForSegments( QListViewItem* item, int segs )
{
    for (int i=0; i<8; i++)
    {
        item->setText(i+1, segs & ( 1<<i )? "1" : "0");
    }
}

QString KLSevenSegmentWidget::formStr( int nr ) const
{
    QString retVal;
    retVal = QString("%1").arg(nr, 0, 16);
    if( retVal.length() == 1 )
        retVal = "0" + retVal;
    return "0x" + retVal;
}

QValueList< int > KLSevenSegmentWidget::generateBitmasks( )
{
    QValueList< int > retVal;
    int current;
    QListViewItem *item=lvSegments->firstChild();
    
    while(item)
    {
        current=0;
        int segList = getSegmentsForListItem( item );
        for (int i=0; i<8; i++)
        {
            if ( segList&(1<<i) )
            {
                current |= m_segmentIsBit[i];
            }
        }
        retVal.append( current );
        item=item->nextSibling();
    }
    return retVal;
}

KLSevenSegmentWidget::~ KLSevenSegmentWidget( )
{
}


#include "klsevensegmentwidget.moc"
